import type {QueryCommandInput} from '@aws-sdk/client-dynamodb'
import {marshall} from '@aws-sdk/util-dynamodb'
import type {DDBReadOnlyClientConfiguration} from '../DDBReadOnlyClient.ts'

export const mapGetItemIndexCommand = <
  Item extends object,
  PKName extends keyof Item,
  SKName extends keyof Item | never = never,
>(
  config: DDBReadOnlyClientConfiguration<Item, PKName, SKName>,
  partitionKey: Item[PKName],
  sortKey?: Item[SKName],
): QueryCommandInput => {
  const partitionKeyName = String(config.partitionKeyName)
  const sortKeyName = String(config.sortKeyName)
  return {
    TableName: config.tableName,
    IndexName: config.indexName,
    KeyConditionExpression: sortKey
      ? `${partitionKeyName} = :partitionKey and ${sortKeyName} = :sortKey`
      : `${partitionKeyName} = :partitionKey`,
    ExpressionAttributeValues: sortKey
      ? marshall({':partitionKey': partitionKey, ':sortKey': sortKey}, {removeUndefinedValues: true})
      : marshall({':partitionKey': partitionKey}, {removeUndefinedValues: true}),
  }
}
