import type {AttributeValue, QueryCommandInput} from '@aws-sdk/client-dynamodb'
import {marshall} from '@aws-sdk/util-dynamodb'
import type {DDBReadOnlyClientConfiguration} from '../DDBReadOnlyClient.ts'

export const mapGetItemsCommand = <
  Item extends object,
  PKName extends keyof Item,
  SKName extends keyof Item | never = never,
>(
  config: DDBReadOnlyClientConfiguration<Item, PKName, SKName>,
  partitionKey: Item[PKName],
  sortKeyPrefix?: string,
  pageSize = 100,
  lastEvaluatedKey?: Record<string, AttributeValue>,
): QueryCommandInput => {
  const partitionKeyName = String(config.partitionKeyName)
  const sortKeyName = String(config.sortKeyName)
  return {
    TableName: config.tableName,
    IndexName: config.indexName,
    KeyConditionExpression: sortKeyPrefix
      ? `${partitionKeyName} = :partitionKey and begins_with(${sortKeyName}, :sortKeyPrefix)`
      : `${partitionKeyName} = :partitionKey`,
    ExpressionAttributeValues: sortKeyPrefix
      ? marshall({':partitionKey': partitionKey, ':sortKeyPrefix': sortKeyPrefix}, {removeUndefinedValues: true})
      : marshall({':partitionKey': partitionKey}, {removeUndefinedValues: true}),
    Limit: pageSize,
    ExclusiveStartKey: lastEvaluatedKey,
  }
}
